# Copyright 2013 tsuru-circus authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from unittest import TestCase
from mock import Mock
import os

from tsuru.hooks import load_config


class LoadConfTest(TestCase):
    def setUp(self):
        self.working_dir = os.path.dirname(__file__)
        self.watcher = Mock(working_dir=self.working_dir)
        self.data = u"""hooks:
  pre-restart:
    - testdata/pre.sh
  post-restart:
    - testdata/pos.sh"""

    def test_load_app_yaml(self):
        yaml_file = os.path.join(self.working_dir, "app.yaml")
        with open(yaml_file, "w") as f:
            f.write(self.data)
        config = load_config(watcher=self.watcher)
        expected = {
            'hooks': {
                'pre-restart': ['testdata/pre.sh'],
                'post-restart': ['testdata/pos.sh'],
            }
        }
        self.assertDictEqual(config, expected)
        os.remove(yaml_file)

    def test_load_app_yml(self):
        yml_file = os.path.join(self.working_dir, "app.yaml")
        with open(yml_file, "w") as f:
            f.write(self.data)
        config = load_config(watcher=self.watcher)
        expected = {
            'hooks': {
                'pre-restart': ['testdata/pre.sh'],
                'post-restart': ['testdata/pos.sh'],
            }
        }
        self.assertDictEqual(config, expected)
        os.remove(yml_file)

    def test_load_without_app_files(self):
        config = load_config(watcher=self.watcher)
        expected = {}
        self.assertDictEqual(config, expected)



# Copyright 2013 tsuru-circus authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from unittest import TestCase
from mock import Mock, patch
import json
import os.path

from tsuru.plugins import ProcfileWatcher

from honcho.procfile import Procfile


class ProcfileWatcherTest(TestCase):
    def build_options(self, name, cmd):
        return json.dumps({
            "command": "add",
            "properties": {
                "cmd": cmd,
                "name": name,
                "args": [],
                "options": {
                    "env": {"port": "8888", "PORT": "8888"},
                    "copy_env": True,
                    "working_dir": "/home/application/current",
                    "stderr_stream": {
                        "class": "tsuru.stream.Stream",
                        "watcher_name": name
                    },
                    "stdout_stream": {
                        "class": "tsuru.stream.Stream",
                        "watcher_name": name
                    },
                    "uid": "ubuntu",
                },
                "start": True,
                "priority": 2,
            },
        })

    def test_add_watcher(self):
        plugin = ProcfileWatcher("", "", 1)
        plugin.envs = lambda: {}
        plugin.circus_client = Mock()
        name = "name"
        cmd = "cmd"
        options = self.build_options(name, cmd)
        plugin.add_watcher(name=name, cmd=cmd)
        plugin.circus_client.call.assert_called_with(options)

    def test_remove_watcher(self):
        plugin = ProcfileWatcher("", "", 1)
        plugin.call = Mock()
        name = "name"
        plugin.remove_watcher(name=name)
        plugin.call.assert_called_with("rm", name=name)

    def test_commands(self):
        plugin = ProcfileWatcher("", "", 1)
        result = Mock()
        result.return_value = {"statuses": {}}
        plugin.call = result
        procfile = Procfile('web: gunicorn -b 0.0.0.0:8080 abyss.wsgi\n')
        to_add, to_remove, to_change = plugin.commands(procfile)
        plugin.call.assert_called_with("status")

    def test_commands_ignore_plugin(self):
        result = Mock()
        result.return_value = {"statuses":
                               {"plugin:tsuru-circus-ProcfileWatcher": "ok"}}
        plugin = ProcfileWatcher("", "", 1)
        plugin.call = result
        procfile = Procfile('web: gunicorn -b 0.0.0.0:8080 abyss.wsgi\n')
        to_add, to_remove, to_change = plugin.commands(procfile)
        self.assertEqual(set(["web"]), to_add)
        self.assertEqual(set([]), to_remove)
        self.assertEqual({}, to_change)

    def test_reload_procfile_add_new_cmds(self):
        plugin = ProcfileWatcher("", "", 1)
        plugin.procfile_path = os.path.join(os.path.dirname(__file__),
                                            "testdata/Procfile1")
        plugin.circus_client = Mock()
        plugin.envs = lambda: {}
        plugin.call = Mock()
        plugin.call.return_value = {"statuses": {}}
        plugin.reload_procfile()
        options = self.build_options("name", "cmd")
        plugin.circus_client.call.assert_called_with(options)

    def test_reload_procfile_remove_old_cmds(self):
        plugin = ProcfileWatcher("", "", 1)
        plugin.call = Mock()
        plugin.call.return_value = {"statuses": {"name": "name", "cmd": "cmd"}}
        plugin.procfile_path = os.path.join(os.path.dirname(__file__),
                                            "testdata/Procfile2")
        plugin.reload_procfile()
        plugin.call.assert_called_with("rm", name="name")

    def test_reload_procfile_update_cmds(self):
        plugin = ProcfileWatcher("", "", 1)
        plugin.call = Mock()
        plugin.call.return_value = {"statuses": {"name": "name", "cmd": "cmd"}}
        plugin.get_cmd = Mock()
        plugin.get_cmd.return_value = "cmd"
        plugin.procfile_path = os.path.join(os.path.dirname(__file__),
                                            "testdata/Procfile3")
        plugin.reload_procfile()
        plugin.get_cmd.assert_called_with("name")
        plugin.call.assert_called_with("set",
                                       name="name",
                                       options={'cmd': 'cmd2'})

    def test_get_cmd(self):
        plugin = ProcfileWatcher("", "", 1)
        plugin.call = Mock()
        plugin.call.return_value = {"options": {"cmd": "ble"}}
        cmd = plugin.get_cmd("name")
        self.assertEqual("ble", cmd)
        plugin.call.assert_called_with("get",
                                       name="name",
                                       keys=["cmd"])

    def test_should_replace_cmds_with_environ(self):
        plugin = ProcfileWatcher("", "", 1)
        plugin.envs = lambda: {}
        plugin.circus_client = Mock()
        name = "name"
        cmd = "echo $port"
        options = self.build_options(name, "echo 8888")
        plugin.add_watcher(name=name, cmd=cmd)
        plugin.circus_client.call.assert_called_with(options)

    def test_should_replace_cmds_with_environ_upercase(self):
        plugin = ProcfileWatcher("", "", 1)
        plugin.envs = lambda: {}
        plugin.circus_client = Mock()
        name = "name"
        cmd = "echo $PORT"
        options = self.build_options(name, "echo 8888")
        plugin.add_watcher(name=name, cmd=cmd)
        plugin.circus_client.call.assert_called_with(options)

    def test_should_replace_cmds_with_environ_with_braces(self):
        plugin = ProcfileWatcher("", "", 1)
        plugin.envs = lambda: {}
        plugin.circus_client = Mock()
        name = "name"
        cmd = "echo ${PORT}"
        options = self.build_options(name, "echo 8888")
        plugin.add_watcher(name=name, cmd=cmd)
        plugin.circus_client.call.assert_called_with(options)

    @patch("tsuru.common.load_envs")
    def test_change_cmd_should_replace_cmds_with_environ(self, load_envs):
        load_envs.return_value = {"BLE": "bla"}
        plugin = ProcfileWatcher("", "", 1)
        plugin.envs = lambda: {}
        plugin.call = Mock()
        plugin.port = 8888
        name = "name"
        cmd = "echo ${PORT} ${BLE}"
        plugin.change_cmd(name=name, cmd=cmd)
        plugin.call.assert_called_with('set',
                                       options={'cmd': 'echo 8888 bla'},
                                       name=name)

    def test_commands_dont_remove_tsuru_hooks_watcher(self):
        plugin = ProcfileWatcher("", "", 1)
        result = Mock()
        result.return_value = {"statuses": {"tsuru-hooks": ""}}
        plugin.call = result
        procfile = Procfile('web: gunicorn -b 0.0.0.0:8080 abyss.wsgi\n')
        to_add, to_remove, to_change = plugin.commands(procfile)
        self.assertNotIn("tsuru-hooks", to_remove)



# Copyright 2013 tsuru-circus authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import unittest

import mock

from tsuru.hooks import set_uid


class Bag(object):
    pass


class SetUIDTestCase(unittest.TestCase):

    @mock.patch("pwd.getpwnam")
    @mock.patch("os.setuid")
    def test_set_uid_from_watcher(self, setuid, getpwnam):
        passwd = Bag()
        passwd.pw_uid = 500
        watcher = Bag()
        watcher.uid = "ubuntu"
        getpwnam.return_value = passwd
        set_uid(watcher)
        getpwnam.assert_called_with("ubuntu")
        setuid.assert_called_with(500)



# Copyright 2013 tsuru-circus authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import os
import pwd
import subprocess

import yaml


def load_config(**kwargs):
    watcher = kwargs.get("watcher")
    files_name = ["app.yaml", "app.yml"]
    for file_name in files_name:
        try:
            with open(os.path.join(watcher.working_dir, file_name)) as f:
                return yaml.load(f.read())
        except IOError:
            pass
    return {}


def set_uid(watcher):
    uid = pwd.getpwnam(watcher.uid).pw_uid
    os.setuid(uid)


def run_commands(name, **kwargs):
    from tsuru.stream import Stream
    config = load_config(**kwargs)
    watcher = kwargs.get("watcher")
    set_uid(watcher)
    cmds = config.get('hooks', {}).get(name, [])
    if cmds:
        Stream(watcher_name=watcher.name)(
            {"data": " ---> Running {}".format(name)})
    for command in cmds:
        try:
            commands = ["/bin/bash", "-c"]
            cmd = "source /home/application/apprc && cd {} && {}"
            commands.append(cmd.format(watcher.working_dir, command))
            result = subprocess.check_output(commands,
                                             stderr=subprocess.STDOUT)
            Stream(watcher_name=watcher.name)({"data": result})
        except subprocess.CalledProcessError as e:
            Stream(watcher_name=watcher.name)({"data": str(e)})
            Stream(watcher_name=watcher.name)({"data": e.output})
    return True


def before_start(*args, **kwargs):
    return run_commands('pre-restart', **kwargs)


def after_start(*args, **kwargs):
    return run_commands('post-restart', **kwargs)



# Copyright 2013 tsuru-circus authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from unittest import TestCase
from mock import patch

from tsuru.hooks import before_start


class BeforeStartTest(TestCase):
    @patch("tsuru.hooks.run_commands")
    def test_before_start(self, run_commands):
        run_commands.return_value = True
        result = before_start()
        run_commands.assert_called_with('pre-restart')
        self.assertTrue(result)



# Copyright 2013 tsuru-circus authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from unittest import TestCase
from mock import patch, call, Mock
import os
import subprocess

from tsuru.hooks import run_commands


class RunCommandsTest(TestCase):
    def setUp(self):
        working_dir = os.path.dirname(__file__)
        self.watcher = Mock(name="somename", working_dir=working_dir,
                            uid="ubuntu")

    def cmd(self, command):
        cmd = "source /home/application/apprc && cd {} && {}"
        return ["/bin/bash", "-c",
                cmd.format(self.watcher.working_dir, command)]

    @patch("tsuru.hooks.load_config")
    @patch("subprocess.check_output")
    @patch("tsuru.hooks.set_uid")
    def test_run_commands_with_config(self, set_uid, check_output, load_config):
        load_config.return_value = {
            'hooks': {
                'pre-restart': ['testdata/pre.sh'],
            }
        }
        run_commands('pre-restart', watcher=self.watcher)
        check_output.assert_called_with(self.cmd("testdata/pre.sh"),
                                        stderr=subprocess.STDOUT)

    @patch("tsuru.hooks.load_config")
    @patch("subprocess.check_output")
    @patch("tsuru.hooks.set_uid")
    def test_run_commands_without_config(self, set_uid, check_output,
                                         load_config):
        load_config.return_value = {
            'hooks': {'pre-restart': []}
        }
        run_commands('pre-restart', watcher=self.watcher)
        self.assertFalse(check_output.called)

    @patch("tsuru.hooks.load_config")
    @patch("subprocess.check_output")
    @patch("tsuru.hooks.set_uid")
    def test_run_commands_many_commands(self, set_uid, check_output, load_config):
        load_config.return_value = {
            'hooks': {
                'pre-restart': ['testdata/pre.sh', 'testdata/pre2.sh'],
            }
        }
        run_commands('pre-restart', watcher=self.watcher)
        calls = [call(self.cmd("testdata/pre.sh"), stderr=subprocess.STDOUT),
                 call(self.cmd("testdata/pre2.sh"), stderr=subprocess.STDOUT)]
        check_output.assert_has_calls(calls)

    @patch("tsuru.hooks.load_config")
    @patch("subprocess.check_output")
    @patch("tsuru.stream.Stream")
    @patch("tsuru.hooks.set_uid")
    def test_run_commands_should_log(self, set_uid, Stream,
                                     check_output, load_config):
        load_config.return_value = {
            'hooks': {
                'pre-restart': ['testdata/pre.sh'],
            }
        }
        check_output.return_value = "ble"
        stream = Stream.return_value
        run_commands('pre-restart', watcher=self.watcher)
        calls = [call({"data": " ---> Running pre-restart"}),
                 call({"data": "ble"})]
        stream.assert_has_calls(calls)

    @patch("tsuru.hooks.load_config")
    @patch("subprocess.check_output")
    @patch("tsuru.stream.Stream")
    @patch("tsuru.hooks.set_uid")
    def test_should_return_true(self, set_uid, Stream, check_output,
                                load_config):
        result = run_commands('pre-restart', watcher=self.watcher)
        self.assertTrue(result)

    @patch("tsuru.hooks.load_config")
    @patch("subprocess.check_output")
    @patch("tsuru.stream.Stream")
    @patch("tsuru.hooks.set_uid")
    def test_run_commands_config_without_a_hook(self, set_uid, Stream,
                                                check_output, load_config):
        load_config.return_value = {
            'hooks': {
                'pre-restart': ['testdata/pre.sh'],
            }
        }
        run_commands('post-restart', watcher=self.watcher)

    @patch("tsuru.hooks.load_config")
    @patch("subprocess.check_output")
    @patch("tsuru.stream.Stream")
    @patch("tsuru.hooks.set_uid")
    def test_run_commands_config_without_hooks(self, set_uid, Stream,
                                               check_output, load_config):
        load_config.return_value = {}
        run_commands('post-restart', watcher=self.watcher)

    @patch("tsuru.hooks.load_config")
    @patch("subprocess.check_output")
    @patch("tsuru.stream.Stream")
    @patch("tsuru.hooks.set_uid")
    def test_run_commands_that_returns_errors(self, set_uid, Stream,
                                              check_output, load_config):
        load_config.return_value = {
            'hooks': {
                'pre-restart': ['exit 1'],
            }
        }
        stream = Stream.return_value
        check_output.side_effect = subprocess.CalledProcessError(1, "cmd")
        run_commands('pre-restart', watcher=self.watcher)
        calls = [
            call({'data': ' ---> Running pre-restart'}),
            call({'data': "Command 'cmd' returned non-zero exit status 1"}),
            call({'data': None})
        ]
        stream.assert_has_calls(calls)

    @patch("tsuru.hooks.load_config")
    @patch("tsuru.stream.Stream")
    @patch("tsuru.hooks.set_uid")
    def test_log_only_when_the_commands_is_executed(self, set_uid, Stream, load_config):
        load_config.return_value = {
            'hooks': {
                'pre-restart': ['ps -ef'],
            }
        }
        stream = Stream.return_value
        run_commands('post-restart', watcher=self.watcher)
        self.assertFalse(stream.called)



# Copyright 2013 tsuru-circus authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from setuptools import setup, find_packages
from tsuru import __version__


setup(
    name="tsuru-circus",
    version=__version__,
    packages=find_packages(),
    description="Circus extensions for tsuru.",
    author="timeredbull",
    author_email="timeredbull@corp.globo.com",
    include_package_data=True,
    install_requires=["circus>=0.6.0", "requests", "honcho==0.2.0", "PyYAML==3.10"],
    tests_require=["nose", "mock"],
    test_suite="nose.collector"
)



import os


def load_envs(path):
    envs = {}
    if os.path.exists(path):
        with open(path) as file:
            for line in file.readlines():
                if "export" in line:
                    line = line.replace("export ", "")
                    k, v = line.split("=")
                    v = v.replace("\n", "").replace('"', '')
                    envs[k] = v
    return envs
